package PDF::Builder::Basic::PDF;
# this file only needed so that HTML documentation with
# multiple descendants can be accommodated.

use strict;
use warnings;

our $VERSION = '3.028'; # VERSION
our $LAST_UPDATE = '3.027'; # manually update whenever code is changed

=head1 NAME

PDF::Builder::Basic::PDF - Various utilities and support routines

=cut

1;
